 #ifndef _SIM_UTILS_H_
#define _SIM_UTILS_H_

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "Fusion_SLAM_types.h"
#include "TueObjFusn_ErrorCodes.h"
#include "fusion_ext.h"

/*! @def EGO_VEHICLE_FAKE_DATA_LEN
    @brief a constant showing the number of rows of the ego vehicle fake data
*/
/*! @def SENSORS_FAKE_DATA_LEN
    @brief a constant showing the number of rows of the sensors fake data
*/
/*! @def LINE_BUFF_SIZE
    @brief size of the buffer for reading a line
*/
/*! @def CSV_ELEMENT_SIZE
    @brief  size of the buffer for reading each element of the CSV file
*/

// TODO: for using matlab-gen dataset, set EGO_VEHICLE_FAKE_DATA_LEN , SENSORS_FAKE_DATA_LEN to 144u
// for using Minieye dataset, set EGO_VEHICLE_FAKE_DATA_LEN , SENSORS_FAKE_DATA_LEN according to info.txt in its folder
#define EGO_VEHICLE_FAKE_DATA_LEN   9999u
#define SENSORS_FAKE_DATA_LEN       3276u 
#define LINE_BUFF_SIZE      65536
#define CSV_ELEMENT_SIZE    32

/**
 * @brief       row of a CSV file datatyp
 * 
 * One row of a csv file is represented with this structure
*/
typedef struct{
    char** elements; /*!< list of strings corresponding to elements of the row*/
    size_t len;      /*!< number of elements in CSVRow (No. columns)*/
} CSVRow;

/**
 * @brief       simulation timestamp struct
 * 
 * Holds system simulation timestamp column of 
 * fake data. Used in timing cycle
*/
typedef struct{
    uint32* timestamp; /*!< integer array holding timestamp values*/
    uint32 idx; /*!< index of current element*/
    uint32 len; /*!< timestamp array length*/
} SimTimestamp;

extern SimTimestamp fusion_cycle_ts;
extern SimTimestamp ego_veh_ts;
// extern SimTimestamp radar_ts;
extern SimTimestamp lidar_ts;
extern SimTimestamp camera_ts;

boolean isDataFinished(const SimTimestamp* const ts);
void checkFinishedStatus(const SimTimestamp* const ts_arr, const uint32 size, boolean* const status);
uint32 getMinTsWithMask(const SimTimestamp* const ts_arr, const uint32 size, const boolean* const fin_status);

void readEgoVehicleData(char* file_name, VehDyn_t* veh_data, size_t cols);
// void readRadarSensorData(char* file_name, ExtObjectList_t* radar_data_ptr, size_t cols);
void readLidarSensorData(char* file_name, Lidar_input_lists* radar_data_ptr, size_t cols);
void readCameraSensorData(char* file_name, Camera_input_lists* cam_data_ptr, size_t cols);
void readSimTimestamp(char* file_name, SimTimestamp* const ts);

void writeFusionOutputCols(FILE** fptr);
void writeFusionOutputToFile(FILE** fptr, const ObjectList_t* const data);

uint32 initSimTimestamp(SimTimestamp* const sim_timestamp, const uint32 size);

#endif //_SIM_UTILS_H_
